#!/usr/bin/perl -w -I../../lib

# Copyright 2007 Adrian Chadd; all rights reserved.

# $Id: graph-summary.cgi,v 1.9 2007/08/22 15:12:17 adrian Exp $

use strict;
use RRDs;
use IO::File;
use SNMP_Session;
use SNMP_util;
use CGI;
use Config::IniFiles;

my ($inicfg);
my ($cfgfile) = "/usr/local/etc/squidstats//config.ini";


my %ranges = ( "1d" => 1, "2d" => 1, "1w" => 1, "30d" => 1, "365d" => 1, "10y" => 1, "all" => 1 );
my %niceid = ( "1d" => "Day", "2d" => "Two Day", "1w" => "Week", "30d" => "Month", "365d" => "Year", "10y" => "Ten Year", "all" => "All Times" );

sub validRange($)
{
        my ($r) = @_;
        return (exists $ranges{$r});
}

sub getNiceIds()
{
        return \%niceid;
}

sub do_graph($$)
{
	my ($template, $range) = @_;
	if (! validRange($range)) {
		return 0;
	}

	# Setup template substitution details - size, range, id, rrd path, graph destination
	my (%subst);
	$subst{"id"} = $range;
	$subst{"width"} = "500";
	$subst{"height"} = "150";
	$subst{"range"} = "-" . $range;
	$subst{"rrdpath"} = $inicfg->val("graph", "database_file");
	$subst{"niceid"} = $niceid{$range};
	$range =~ s/[\/\.]//g;
	$subst{"graph"} =  $inicfg->val("graph", "graph_datadir") . "/" . $template . "." . $range . ".png";

	# templates are in a known directory!
	my ($f) = new IO::File;
	my ($fname) = $template;
	$template =~ s/[\/\.]//g;
	if (! $f->open($inicfg->val("graph", "template_dir"). "/" . $template)) {
		print "ERROR: couldn't generate graph ($template / $range): $!\n";
		return 0;
	}

	# The template 'just' has the graph parameters to pass to rrdtool, one
	# line per parameter.
	my (@p);
	foreach (<$f>) {
		chomp;
		next if /^$/;
		next if /^#/;
		s/%([A-Za-z0-9_]*?)%/$subst{$1}/ge;
		# print "SUBST: '$_'\n";
		push @p, $_;
	}

	RRDs::graph($subst{"graph"}, "-w", $subst{"width"}, "-h", $subst{"height"}, "-E", @p);
	if (defined RRDs::error) {
		print "ERROR:" . RRDs::error . "\n";
	} else {
		return 0;
	}
}


sub display_graph($$)
{
	my ($graph, $range) = @_;

	my (@r) = ( "1d", "2d", "1w", "30d", "365d", "10y" );
	# Filter graph/range strings
       	# Generate the four sub-graphs
       	foreach (@r) {
		if ($range eq "all" || $_ eq $range) {
               		print "<tr>\n";
			print "  <td class='sectionAttrib'>\n";
			do_graph($graph, $_);
			print "<img src='" . $inicfg->val("graph", "graph_webdir") . "/" . $graph . "." . $_ . ".png' border='0'><br>\n";
			print "  </td>\n";
               		print "</tr>\n";
		}
	}
}

my ($q) = new CGI;

my ($refresh_time) = $q->param("refresh_time");
my ($graph) = $q->param("graph");
my ($range) = $q->param("range");

if (! defined $range) {
	$range = "1d";
}
if (! defined $refresh_time) {
	$refresh_time = 300;
}
my (%rtime) = ( "300" => "5 Minutes", "600" => "10 Minutes", "900" => "15 Minutes");
if (! exists $rtime{$refresh_time}) {
	$refresh_time = 300;
}

# Don't allow invalid ranges
if (! validRange($range)) {
	# XXX eww, but hackers suck!
	$range = "1d";
}

# Don't allow invalid graphs
if (! defined $graph || $graph =~ m/[\.\/]/) {
	$graph = "incoming";
}


print $q->header();
$inicfg = new Config::IniFiles( -file => "$cfgfile" ) || die "Couldn't open $cfgfile: $!\n";
print $q->start_html(-title => 'Graph Summary', -style => { src=> $inicfg->val("graph", "css_webfile") } );
print '<meta http-equiv="refresh" content="' . $refresh_time . ";URL='graph-summary.cgi?graph=$graph&range=$range&refresh_time=$refresh_time'" . '">' . "\n";

my (@templates);
my (%template_names);
my ($fh) = new IO::File;
$fh->open($inicfg->val("graph", "template_conf")) || die "[Couldn't read templates config file]";
foreach (<$fh>)
{
	chomp;
	# print "GOT: $_\n";
	m/^(.*?)[\s\t]*"(.*)"$/;
	# print "Template: $1 -> $2\n";
	unshift @templates, $1;
	$template_names{$1} = $2;
}
$template_names{"all"} = "All Graphs";
$fh->close;
undef $fh;

print "<table class='statusTable'>\n";
print "<tr>\n";
print "  <td class='sectionHeader'>Graph Overview</td>\n";
print "</tr>\n";

print "<tr>\n";
print "  <td class='sectionAttrib'>\n";
print $q->start_form(-action=> 'graph-summary.cgi', -method=>'submit') . "\n";
print "Time:" . $q->popup_menu(-name => 'range', -values => [ 'all', '1d', '2d', '1w', '30d', '365d', '10y' ], -default => $range, -labels => getNiceIds());
print "Graph:" . $q->popup_menu(-name => 'graph', -values => [ "all", @templates ], -default => $graph, -labels => \%template_names);
print "Refresh:" . $q->popup_menu(-name => 'refresh_time', -values => [ '300', '600', '900'], -default => $refresh_time, -labels => \%rtime);
print "<input type='submit' name='select' value='select'>\n";
print $q->end_form() . "\n";
print "  </td>\n";
print "</tr>\n";

# present the user with the ability to choose a range and a graph; 
# [all] is a valid graph type and will display all of the graphs


foreach (@templates) {
	if ($graph eq "all" || $graph eq $_) {
		print "<tr><td class='sectionHeader'>" . $q->escapeHTML($template_names{$_}) . "</td></tr>\n";
		display_graph($_, $range);
	}
}
print "</table>\n";
print $q->end_html();

exit 0;
